technique T0
{
    pass P0
    {
		// Disable specular
		SpecularEnable = False;
		
		// Set Z-buffering
		ZEnable			= True;
		ZWriteEnable	= True;
		
		// Set alpha blending mode
		AlphaBlendEnable = False;
		
		// Set texture address mode and min mag filters
		AddressU[0]		= Wrap;
		AddressV[0]		= Wrap;
		MipFilter[0]	= Linear;
		MinFilter[0]	= Linear;
		MagFilter[0]	= Linear;
		
		AddressU[1]		= Wrap;
		AddressV[1]		= Wrap;
		MipFilter[1]	= Linear;
		MinFilter[1]	= Linear;
		MagFilter[1]	= Linear;
		
		// Set shadow buffer args
        AddressU[2]		= Clamp;
		AddressV[2]		= Clamp;
		MipFilter[2]	= Linear;
		MinFilter[2]	= Linear;
		MagFilter[2]	= Linear;
				
		// Set shadow buffer args
        AddressU[3]		= Clamp;
		AddressV[3]		= Clamp;
		MipFilter[3]	= Linear;
		MinFilter[3]	= Linear;
		MagFilter[3]	= Linear;
	
		VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            float v3[3];	// Normal
            stream 1;
            float v7[2];	// UV
            stream 2;
            float v8[2];	// UV
        }
		asm
		{
			vs.1.1

			// Transform position
			// Apply view+proj
			m4x4	r0, v0, c92
			mov		oPos, r0
			mov		oFog.x, r0.w
			
			// Lighting (diffuse + ambient + emissive)
			dp3		r1, v3, -c81			// Dot for diffuse
			lit		r1, r1
			mul		oD0, r1.yyyw, c83
			
			// Move the uvs through...
			add		oT0, v7, c60
			add		oT1, v8, c61
			
			// Compute uvs for shadow buffers
			m4x4	r2, v0, c76
			max		oT2, r2, c86.wwwz	// Clamps W to be >= 0.0 (xyz should pass through)
			
			// Compute uvs for shadow buffers
			m4x4	r3, v0, c72
			max		oT3, r3, c86.wwwz	// Clamps W to be >= 0.0 (xyz should pass through)
		};
		
		PixelShader = 
        asm
        {
			// Declare pixel shader version 1.1
			ps.1.1

			// Define textures
			tex t0
			tex t1
			tex t2
			tex t3

			mul r0.rgb, v0, t2
			+add t3.a, t3.b, c0.a
			mad r0.rgb, r0, t3.a, c0
			+mov r0.a, v0.a
			lrp r1, t1.a, t1, t0
			mul r0, r0, r1
        };    
    }
}

technique T1
{
    pass P0
    {
		// Disable specular
		SpecularEnable = False;
		
		// Set Z-buffering
		ZEnable			= True;
		ZWriteEnable	= True;
		
		// Set alpha blending mode
		AlphaBlendEnable = False;
		
		// Set texture address mode and min mag filters
		AddressU[0]		= Wrap;
		AddressV[0]		= Wrap;
		MinFilter[0]	= Linear;
		MagFilter[0]	= Linear;
		
		AddressU[1]		= Wrap;
		AddressV[1]		= Wrap;
		MinFilter[1]	= Linear;
		MagFilter[1]	= Linear;
		
        // First stage
        ColorOp[0]   = SelectArg1;
        ColorArg1[0] = Texture;
        ColorArg2[0] = Texture;
        AlphaOp[0]   = SelectArg1;
        AlphaArg1[0] = Texture;
        AlphaArg2[0] = Texture;
        
        // Second stage
        ColorOp[1]   = BlendTextureAlpha;
        ColorArg1[1] = Texture;
        ColorArg2[1] = Current;
        AlphaOp[1]   = SelectArg1;
        AlphaArg1[1] = Texture;
        AlphaArg2[1] = Current;
        
        // Third stage
        ColorOp[2]   = Modulate;
        ColorArg1[2] = Diffuse;
        ColorArg2[2] = Texture;
        AlphaOp[2]   = Disable;
        
        // Fourth stage
        ColorOp[3]   = Disable;
         
		VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            float v3[3];	// Normal
            stream 1;
            float v7[2];	// UV
            stream 2;
            float v8[2];	// UV
        };     
    }
}
