technique T0
{
	// modulate pass
    pass P0
    {
		// Disable specular
		SpecularEnable = False;
		
		// Set Z-buffering
		ZEnable			= True;
		ZWriteEnable	= True;
		
		// Set alpha blending mode
		AlphaBlendEnable = True;
		SrcBlend		= SrcAlpha;
		DestBlend		= InvSrcAlpha;
		
 		AddressU[0]		= Clamp;
		AddressV[0]		= Wrap;
		MipFilter[0]	= None;
		MinFilter[0]	= Linear;
		MagFilter[0]	= Linear;
		
		AddressU[1]		= Wrap;
		AddressV[1]		= Wrap;
		MipFilter[1]	= Linear;
		MinFilter[1]	= Linear;
		MagFilter[1]	= Linear;
		
		AddressU[2]		= Wrap;
		AddressV[2]		= Wrap;
		MipFilter[2]	= Linear;
		MinFilter[2]	= Linear;
		MagFilter[2]	= Linear;
		
		VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            D3DCOLOR v3;	// Normal
            stream 1;
            short v7[2];	// UV
            stream 2;
            short v8[2];	// UV
            stream 3;
            short v9[2];	// UV
        }
		asm
		{
			vs.1.1

			// Transform position
			// Apply view+proj
			m4x4	oPos, v0, c92
			slt		oFog.x, v0, v0
			
			// No Lighting
			
			// Move the uvs through...
			mul		r0, v7, c91.x
			add		oT0.xyz, r0, c60
			mul		r0, v8, c91.x
			add		oT1.xyz, r0, c61
			mul		r0, v9, c91.x
			add		oT2.xyz, r0, c62
			
		};
		
		PixelShader = 
        asm
        {
			// Declare pixel shader version 1.1
			ps.1.1

			// Define textures
			tex t0
			tex t1
			tex t2

			mul_x2 r0, t1, t2
			mul_x2 r0.rgb, r0, t0
			+mul r0.a, r0, t0
        };     
    }
    
    // lighting pass
    pass P1
    {
		// Disable specular
		SpecularEnable = False;
		
		// Set alpha blending mode
//		AlphaBlendEnable = True;
//		SrcBlend		= Zero;
//		DestBlend		= SrcColor;
		
		// Set shadow map args
 		AddressU[0]		= Clamp;
		AddressV[0]		= Clamp;
		MipFilter[0]	= Linear;
		MinFilter[0]	= Linear;
		MagFilter[0]	= Linear;
	
		// Set shadow buffer args
 		AddressU[1]		= Clamp;
		AddressV[1]		= Clamp;
		MipFilter[1]	= Linear;
		MinFilter[1]	= Linear;
		MagFilter[1]	= Linear;
      
		VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            D3DCOLOR v3;	// Normal
        }
		asm
		{
			vs.1.1

			// Transform position
			// Apply view+proj
			m4x4	oPos, v0, c92
			slt		oFog.x, v0, v0
			
			// Lighting (diffuse + ambient + emissive)
			mad		r0, v3, c91.z, -v3.w	// EXPECTING 1.0 to be in v3.w
			dp3		r1, r0, -c81			// Dot for diffuse
			lit		r1, r1
			mul		oD0, r1.yyyw, c83
			
			// Compute uvs for shadow buffers
			m4x4	r2, v0, c76
			max		oT0, r2, c86.wwwz	// Clamps W to be >= 0.0 (xyz should pass through)
			
			// Compute uvs for shadow buffers
			m4x4	r3, v0, c72
			max		oT1, r3, c86.wwwz	// Clamps W to be >= 0.0 (xyz should pass through)
		};
		
		PixelShader = 
        asm
        {
			// Declare pixel shader version 1.1
			ps.1.1

			// Define textures
			tex t0
			tex t1

			mul r0.rgb, v0, t0
			+add t1.a, t1.b, c0.a
			mad r0.rgb, r0, t1.a, c0
			+mov r0.a, v0.a
        };     
    }
    
    // fog pass
    pass P2
    {
		// Disable specular
		SpecularEnable = False;
		
		// Set alpha blending mode
		AlphaBlendEnable = True;
		SrcBlend		= InvSrcAlpha;
		DestBlend		= SrcAlpha;
	
		VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            D3DCOLOR v3;	// Normal
        }
		asm
		{
			vs.1.1

			// Transform position
			// Apply view+proj
			m4x4	r0, v0, c92
			mov		oPos, r0
			slt		oFog.x, v0, v0

			// Caluculate fog
			mov r1.x, c0.x			// mov base log 2.7 into x
			mul r1.y, r0.w, c0.y	// mul distance by density and put into y
			lit r1.z, r1.xxyy		// take x to the power of y
			rcp oD0.w, r1.z
		};
		
		PixelShader = 
        asm
        {
			// Declare pixel shader version 1.1
			ps.1.1

			// No textures
			mov r0.rgb, c1			// mov fog color into rgb channel
			+mov r0.a, v0.a			// mov fog factor into alpha
        };     
    }
}

